@@ -1,38 +1,3 @@
-# ----------------------------------------------------------------------------
-# pyglet
-# Copyright (c) 2006-2008 Alex Holkner
-# Copyright (c) 2008-2022 pyglet contributors
-# All rights reserved.
-#
-# Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions
-# are met:
-#
-#  * Redistributions of source code must retain the above copyright
-#    notice, this list of conditions and the following disclaimer.
-#  * Redistributions in binary form must reproduce the above copyright
-#    notice, this list of conditions and the following disclaimer in
-#    the documentation and/or other materials provided with the
-#    distribution.
-#  * Neither the name of pyglet nor the names of its
-#    contributors may be used to endorse or promote products
-#    derived from this software without specific prior written
-#    permission.
-#
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
-# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
-# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
-# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
-# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
-# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
-# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
-# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-# POSSIBILITY OF SUCH DAMAGE.
-# ----------------------------------------------------------------------------
-
 """Event dispatch framework.
 
 All objects that produce events in pyglet implement :py:class:`~pyglet.event.EventDispatcher`,
@@ -219,7 +184,7 @@ class EventDispatcher:
                 # Single magically named function
                 name = obj.__name__
                 if name not in self.event_types:
-                    raise EventException('Unknown event "%s"' % name)
+                    raise EventException(f'Unknown event "{name}"')
                 if inspect.ismethod(obj):
                     yield name, WeakMethod(obj, partial(self._remove_handler, name))
                 else:
@@ -234,7 +199,7 @@ class EventDispatcher:
         for name, handler in kwargs.items():
             # Function for handling given event (no magic)
             if name not in self.event_types:
-                raise EventException('Unknown event "%s"' % name)
+                raise EventException(f'Unknown event "{name}"')
             if inspect.ismethod(handler):
                 yield name, WeakMethod(handler, partial(self._remove_handler, name))
             else:
@@ -351,38 +316,45 @@ class EventDispatcher:
         This is normally called from a dead ``WeakMethod`` to remove itself from the
         event stack.
         """
+
         # Iterate over a copy as we might mutate the list
         for frame in list(self._event_stack):
-            if name in frame and frame[name] == handler:
-                del frame[name]
-                if not frame:
-                    self._event_stack.remove(frame)
+
+            if name in frame:
+                try:
+                    if frame[name] == handler:
+                        del frame[name]
+                        if not frame:
+                            self._event_stack.remove(frame)
+                except TypeError:
+                    # weakref is already dead
+                    pass
 
     def dispatch_event(self, event_type, *args):
-        """Dispatch a single event to the attached handlers.
+        """Dispatch an event to the attached event handlers.
 
-        The event is propagated to all handlers from from the top of the stack
-        until one returns `EVENT_HANDLED`.  This method should be used only by
-        :py:class:`~pyglet.event.EventDispatcher` implementors; applications should call
-        the ``dispatch_events`` method.
+        The event is propagated to all registered event handlers
+        in the stack, starting and the top and going down. If any
+        registered event handler returns `EVENT_HANDLED`, no further
+        handlers down the stack will receive this event.
 
-        Since pyglet 1.2, the method returns `EVENT_HANDLED` if an event
-        handler returned `EVENT_HANDLED` or `EVENT_UNHANDLED` if all events
-        returned `EVENT_UNHANDLED`.  If no matching event handlers are in the
-        stack, ``False`` is returned.
+        This method has several possible return values.
+        If any event hander has returned `EVENT_HANDLED`, then this
+        method will also return `EVENT_HANDLED`. If not, this method
+        will return `EVENT_UNHANDLED`. If there were no events
+        registered to receive this event, ``False`` is returned.
 
         :Parameters:
             `event_type` : str
-                Name of the event.
+                The name of the event to dispatch.
             `args` : sequence
-                Arguments to pass to the event handler.
+                Optional arguments to pass to the event handlers.
 
         :rtype: bool or None
-        :return: (Since pyglet 1.2) `EVENT_HANDLED` if an event handler
-            returned `EVENT_HANDLED`; `EVENT_UNHANDLED` if one or more event
-            handlers were invoked but returned only `EVENT_UNHANDLED`;
-            otherwise ``False``.  In pyglet 1.1 and earlier, the return value
-            is always ``None``.
+        :return: `EVENT_HANDLED` if any event handler returned `EVENT_HANDLED`;
+                 `EVENT_UNHANDLED` if one or more event handlers were invoked
+                 without any of them returning `EVENT_HANDLED`;
+                 ``False`` if no event handlers were registered.
 
         """
         assert hasattr(self, 'event_types'), (
@@ -391,7 +363,7 @@ class EventDispatcher:
             "EventDispatcher.register_event_type('event_name')."
         )
         assert event_type in self.event_types, \
-            "%r not found in %r.event_types == %r" % (event_type, self, self.event_types)
+            f"{event_type} not found in {self}.event_types == {self.event_types}"
 
         invoked = False
 
