@@ -18,7 +18,37 @@ class WidgetBase(EventDispatcher):
         self._height = height
         self._bg_group = None
         self._fg_group = None
-        self.enabled = True
+        self._enabled = True
+
+    def _set_enabled(self, enabled: bool) -> None:
+        """Internal hook for setting enabled.
+
+        Override this in subclasses to perform effects when a widget is
+        enabled or disabled.
+        """
+        pass
+
+    @property
+    def enabled(self) -> bool:
+        """Get/set whether this widget is enabled.
+
+        To react to changes in this value, override
+        :py:meth:`._set_enabled` on widgets. For example, you may want
+        to cue the user by:
+
+        * Playing an animation and/or sound
+        * Setting a highlight color
+        * Displaying a toast or notification
+
+        """
+        return self._enabled
+
+    @enabled.setter
+    def enabled(self, new_enabled: bool) -> None:
+        if self._enabled == new_enabled:
+            return
+        self._enabled = new_enabled
+        self._set_enabled(new_enabled)
 
     def update_groups(self, order):
         pass
@@ -110,6 +140,12 @@ class WidgetBase(EventDispatcher):
     def _update_position(self):
         raise NotImplementedError("Unable to reposition this Widget")
 
+    def on_key_press(self, symbol, modifiers):
+        pass
+
+    def on_key_release(self, symbol, modifiers):
+        pass
+
     def on_mouse_press(self, x, y, buttons, modifiers):
         pass
 
@@ -166,7 +202,6 @@ class PushButton(WidgetBase):
         self._depressed_img = depressed
         self._hover_img = hover or depressed
 
-        # TODO: add `draw` method or make Batch required.
         self._batch = batch or pyglet.graphics.Batch()
         self._user_group = group
         bg_group = Group(order=0, parent=group)
@@ -214,6 +249,12 @@ class PushButton(WidgetBase):
             return
         self._sprite.image = self._hover_img if self._check_hit(x, y) else self._depressed_img
 
+    def on_press(self):
+        """Event: Dispatched when the button is clicked."""
+
+    def on_release(self):
+        """Event: Dispatched when the button is released."""
+
 
 PushButton.register_event_type('on_press')
 PushButton.register_event_type('on_release')
@@ -240,6 +281,9 @@ class ToggleButton(PushButton):
             return
         self._sprite.image = self._get_release_image(x, y)
 
+    def on_toggle(self, value: bool):
+        """Event: returns True or False to indicate the current state."""
+
 
 ToggleButton.register_event_type('on_toggle')
 
@@ -359,14 +403,18 @@ class Slider(WidgetBase):
             return
         self._in_update = False
 
+    def on_change(self, value: float):
+        """Event: Returns the current value when the slider is changed."""
+
 
 Slider.register_event_type('on_change')
 
 
 class TextEntry(WidgetBase):
-    """Instance of a text entry widget.
-
-    Allows the user to enter and submit text.
+    """Instance of a text entry widget. Allows the user to enter and submit text.
+    
+    Triggers the event 'on_commit', when the user hits the Enter or Return key.
+    The current text string is passed along with the event.
     """
 
     def __init__(self, text, x, y, width,
@@ -515,10 +563,8 @@ class TextEntry(WidgetBase):
         if self._focus:
             self._caret.on_text_motion_select(motion)
 
-    def on_commit(self, text):
-        if not self.enabled:
-            return
-        """Text has been commited via Enter/Return key."""
+    def on_commit(self, text: str):
+        """Event: dispatches the current text when commited via Enter/Return key."""
 
 
 TextEntry.register_event_type('on_commit')
