@@ -689,7 +689,7 @@ class ImageData(AbstractImage):
         :rtype: cls or cls.region_class
         """
         internalformat = self._get_internalformat(self._desired_format)
-        texture = cls.create(self.width, self.height, GL_TEXTURE_2D, internalformat, False, blank_data=False)
+        texture = cls.create(self.width, self.height, GL_TEXTURE_2D, internalformat, blank_data=False)
         if self.anchor_x or self.anchor_y:
             texture.anchor_x = self.anchor_x
             texture.anchor_y = self.anchor_y
@@ -1224,7 +1224,7 @@ class Texture(AbstractImage):
         """Delete this texture and the memory it occupies.
         After this, it may not be used anymore.
         """
-        glDeleteTextures(1, self.id)
+        glDeleteTextures(1, GLuint(self.id))
         self.id = None
 
     def __del__(self):
@@ -1561,10 +1561,12 @@ class Texture3D(Texture, UniformTextureSequence):
 
     def __setitem__(self, index, value):
         if type(index) is slice:
+            glBindTexture(self.target, self.id)
+
             for item, image in zip(self[index], value):
                 image.blit_to_texture(self.target, self.level, image.anchor_x, image.anchor_y, item.z)
         else:
-            value.blit_to_texture(self.target, self.level, value.anchor_x, value.anchor_y, self[index].z)
+            self.blit_into(value, value.anchor_x, value.anchor_y, self[index].z)
 
     def __iter__(self):
         return iter(self.items)
@@ -1647,7 +1649,8 @@ class TextureArray(Texture, UniformTextureSequence):
         self._verify_size(image)
         start_length = len(self.items)
         item = self.region_class(0, 0, start_length, image.width, image.height, self)
-        image.blit_to_texture(self.target, self.level, image.anchor_x, image.anchor_y, start_length)
+
+        self.blit_into(image, image.anchor_x, image.anchor_y, start_length)
         self.items.append(item)
         return item
 
@@ -1656,6 +1659,8 @@ class TextureArray(Texture, UniformTextureSequence):
         if len(self.items) + len(images) > self.max_depth:
             raise TextureArrayDepthExceeded("The amount of images being added exceeds the depth of this TextureArray.")
 
+        glBindTexture(self.target, self.id)
+
         start_length = len(self.items)
         for i, image in enumerate(images):
             self._verify_size(image)
@@ -1679,6 +1684,8 @@ class TextureArray(Texture, UniformTextureSequence):
 
     def __setitem__(self, index, value):
         if type(index) is slice:
+            glBindTexture(self.target, self.id)
+
             for old_item, image in zip(self[index], value):
                 self._verify_size(image)
                 item = self.region_class(0, 0, old_item.z, image.width, image.height, self)
@@ -1687,7 +1694,7 @@ class TextureArray(Texture, UniformTextureSequence):
         else:
             self._verify_size(value)
             item = self.region_class(0, 0, index, value.width, value.height, self)
-            value.blit_to_texture(self.target, self.level, value.anchor_x, value.anchor_y, index)
+            self.blit_into(value, value.anchor_x, value.anchor_y, index)
             self.items[index] = item
 
     def __iter__(self):
@@ -1743,14 +1750,6 @@ class TileableTexture(Texture):
         return image.create_texture(cls)
 
 
-class DepthTexture(Texture):
-    """A texture with depth samples (typically 24-bit)."""
-
-    def blit_into(self, source, x, y, z):
-        glBindTexture(self.target, self.id)
-        source.blit_to_texture(self.level, x, y, z)
-
-
 class ImageGrid(AbstractImage, AbstractImageSequence):
     """An imaginary grid placed over an image allowing easy access to
     regular regions of that image.
@@ -1982,10 +1981,10 @@ class BufferManager:
     """
 
     def __init__(self):
-        self.color_buffer = None
-        self.depth_buffer = None
+        self._color_buffer = None
+        self._depth_buffer = None
         self.free_stencil_bits = None
-        self.refs = []
+        self._refs = []
 
     @staticmethod
     def get_viewport():
@@ -2006,11 +2005,11 @@ class BufferManager:
         viewport = self.get_viewport()
         viewport_width = viewport[2]
         viewport_height = viewport[3]
-        if (not self.color_buffer or
-                viewport_width != self.color_buffer.width or
-                viewport_height != self.color_buffer.height):
-            self.color_buffer = ColorBufferImage(*viewport)
-        return self.color_buffer
+        if (not self._color_buffer or
+                viewport_width != self._color_buffer.width or
+                viewport_height != self._color_buffer.height):
+            self._color_buffer = ColorBufferImage(*viewport)
+        return self._color_buffer
 
     def get_depth_buffer(self):
         """Get the depth buffer.
@@ -2020,11 +2019,11 @@ class BufferManager:
         viewport = self.get_viewport()
         viewport_width = viewport[2]
         viewport_height = viewport[3]
-        if (not self.depth_buffer or
-                viewport_width != self.depth_buffer.width or
-                viewport_height != self.depth_buffer.height):
-            self.depth_buffer = DepthBufferImage(*viewport)
-        return self.depth_buffer
+        if (not self._depth_buffer or
+                viewport_width != self._depth_buffer.width or
+                viewport_height != self._depth_buffer.height):
+            self._depth_buffer = DepthBufferImage(*viewport)
+        return self._depth_buffer
 
     def get_buffer_mask(self):
         """Get a free bitmask buffer.
@@ -2057,7 +2056,7 @@ class BufferManager:
         def release_buffer(ref, owner=self):
             owner.free_stencil_bits.insert(0, stencil_bit)
 
-        self.refs.append(weakref.ref(bufimg, release_buffer))
+        self._refs.append(weakref.ref(bufimg, release_buffer))
 
         return bufimg
 
@@ -2143,22 +2142,11 @@ class DepthBufferImage(BufferImage):
     """The depth buffer.
     """
     gl_format = GL_DEPTH_COMPONENT
-    format = 'L'
+    format = 'R'
 
     def get_texture(self, rectangle=False):
-        assert rectangle is False, 'Depth textures cannot be rectangular'
-
-        texture = DepthTexture.create(self.width, self.height, GL_TEXTURE_2D, None)
-        if self.anchor_x or self.anchor_y:
-            texture.anchor_x = self.anchor_x
-            texture.anchor_y = self.anchor_y
-
-        glReadBuffer(self.gl_buffer)
-        glCopyTexImage2D(texture.target, 0,
-                         GL_DEPTH_COMPONENT,
-                         self.x, self.y, self.width, self.height,
-                         0)
-        return texture
+        image_data = self.get_image_data()
+        return image_data.get_texture(rectangle)
 
     def blit_to_texture(self, target, level, x, y, z):
         glReadBuffer(self.gl_buffer)
@@ -2169,6 +2157,6 @@ class BufferImageMask(BufferImage):
     """A single bit of the stencil buffer.
     """
     gl_format = GL_STENCIL_INDEX
-    format = 'L'
+    format = 'R'
 
     # TODO mask methods
